@@ -1,14 +1,20 @@
-"""
-Runner for SimulationExperiments.
+"""Runner for SimulationExperiments.
+
+The ExperimentRunner is used to execute simulation experiments.
+This includes
+- loading of datasets
+- loading of models
+- running tasks (simulation on models)
+- creating outputs
 """
 
 import logging
 from pathlib import Path
-from typing import Iterable, List, Tuple
+from typing import Dict, Iterable, List, Optional
 
-from sbmlsim.experiment import SimulationExperiment
+from sbmlsim.experiment import ExperimentResult, SimulationExperiment
 from sbmlsim.model import RoadrunnerSBMLModel
-from sbmlsim.simulator.simulation import SimulatorAbstract
+from sbmlsim.simulator import SimulatorSerial
 from sbmlsim.units import UnitRegistry, Units
 from sbmlsim.utils import timeit
 
@@ -24,7 +30,7 @@ class ExperimentRunner(object):
         experiment_classes: Iterable[SimulationExperiment],
         base_path: Path,
         data_path: Path,
-        simulator: SimulatorAbstract = None,
+        simulator: SimulatorSerial = None,
         ureg: UnitRegistry = None,
         **kwargs,
     ):
@@ -38,30 +44,43 @@ class ExperimentRunner(object):
         # initialize experiments
         self.base_path = base_path
         self.data_path = data_path
-        self.experiments = {}
+        self.experiments: Dict[str, SimulationExperiment] = {}
         self.models = {}
+        self.simulator: Optional[SimulatorSerial] = None
 
         self.initialize(experiment_classes, **kwargs)
         self.set_simulator(simulator)
 
-    def set_simulator(self, simulator):
+    def set_simulator(self, simulator: SimulatorSerial) -> None:
         """Set simulator on the runner and experiments."""
         if simulator is None:
             logger.debug(
-                f"No simulator set in ExperimentRunner. This warning can be "
-                f"ignored for parameter fitting which provides a simulator."
+                "No simulator set in ExperimentRunner. This warning can be "
+                "ignored in parameter fitting."
             )
         else:
-            self.simulator = simulator  # type: SimulatorAbstract
+            self.simulator: SimulatorSerial = simulator
             for experiment in self.experiments.values():
                 experiment.simulator = simulator
 
     @timeit
     def initialize(self, experiment_classes, **kwargs):
+        """Initialize ExperimentRunner.
+
+        Initialization is required in addition to construction to allow serialization
+        of information for parallelization.
+        """
         if not isinstance(experiment_classes, (list, tuple, set)):
             experiment_classes = [experiment_classes]
 
         for exp_class in experiment_classes:
+            if not isinstance(exp_class, type):
+                raise ValueError(
+                    f"All 'experiment_classes' must be a class definition deriving "
+                    f"from 'SimulationExperiment', but '{exp_class}' is "
+                    f"'{type(exp_class)}'."
+                )
+
             logger.debug(f"Initialize SimulationExperiment: {exp_class.__name__}")
             experiment = exp_class(
                 base_path=self.base_path,
@@ -92,7 +111,7 @@ class ExperimentRunner(object):
         save_results: bool = False,
         figure_formats: List[str] = None,
         reduced_selections: bool = True,
-    ) -> List["ExperimentResult"]:
+    ) -> List[ExperimentResult]:
         """Run the experiments."""
         if not output_path.exists():
             output_path.mkdir(parents=True)
