@@ -1,14 +1,17 @@
+"""Analysis of fitting results."""
+
 import datetime
 import json
 import logging
 import time
 import uuid
 from pathlib import Path
-from typing import Dict, Iterable, List, Set, Tuple, Union
+from typing import Dict, Iterable, List, Optional, Set, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import seaborn as sns
+from matplotlib.figure import Figure
 from scipy.optimize import OptimizeResult
 
 from sbmlsim.fit.objects import FitParameter
@@ -21,7 +24,7 @@ logger = logging.getLogger(__name__)
 
 
 class OptimizationResult(ObjectJSONEncoder):
-    """Result of optimization problem"""
+    """Result of optimization problem."""
 
     def __init__(
         self,
@@ -30,7 +33,7 @@ class OptimizationResult(ObjectJSONEncoder):
         trajectories: List,
         sid: str = None,
     ):
-        """Result of an optimization.
+        """Initialize optimization result.
 
         Provides access to the FitParameters, the individual fits, and
         the trajectories of the fits.
@@ -85,12 +88,14 @@ class OptimizationResult(ObjectJSONEncoder):
         d = from_json(json_info)
         return OptimizationResult(**d)
 
-    def __str__(self):
+    def __str__(self) -> str:
+        """Get string representation."""
         info = f"<OptimizationResult: n={self.size}>"
         return info
 
     @staticmethod
-    def combine(opt_results: List[OptimizeResult]):
+    def combine(opt_results: List["OptimizationResult"]) -> "OptimizationResult":
+        """Combine results from multiple parameter fitting experiments."""
         # FIXME: check that the parameters are fitting
         parameters = opt_results[0].parameters
         pids = {p.pid for p in parameters}
@@ -113,7 +118,7 @@ class OptimizationResult(ObjectJSONEncoder):
 
     @property
     def size(self):
-        """Number of optimizations in result."""
+        """Get number of optimization runs in result."""
         return len(self.df_fits)
 
     @property
@@ -127,7 +132,7 @@ class OptimizationResult(ObjectJSONEncoder):
         return self._x_as_fit_parameters(x=self.xopt)
 
     def _x_as_fit_parameters(self, x) -> List[FitParameter]:
-        """Converts numerical parameter vector to fit parameters."""
+        """Convert numerical parameter vector to fit parameters."""
         fit_pars = []
         for k, p in enumerate(self.parameters):
             fit_pars.append(
@@ -190,14 +195,15 @@ class OptimizationResult(ObjectJSONEncoder):
 
         return df
 
-    def report(self, path: Path = None, print_output=True) -> str:
-        """ Readable report of optimization. """
+    def report(self, path: Optional[Path] = None, print_output: bool = True) -> str:
+        """Report of optimization."""
         pd.set_option("display.max_columns", None)
         pd.set_option("display.expand_frame_repr", False)
         info = [
             "\n",
             "-" * 80,
-            f"Optimization Results: {self.sid}",
+            "-" * 80,
+            f"Optimization results: {self.sid}",
             "-" * 80,
             str(self.df_fits),
             "-" * 80,
@@ -240,15 +246,16 @@ class OptimizationResult(ObjectJSONEncoder):
         return info
 
     @staticmethod
-    def _save_fig(fig, path: Path, show_plots: bool = True):
+    def _save_fig(fig: Figure, path: Path, show_plots: bool = True):
         if show_plots:
             plt.show()
         if path:
             fig.savefig(path, bbox_inches="tight")
+        plt.close(fig)
 
     @timeit
     def plot_waterfall(self, path: Path = None, show_plots: bool = True):
-        """Creates waterfall plot for the fit results.
+        """Create waterfall plot for the fit results.
 
         Plots the optimization runs sorted by cost.
         """
@@ -266,8 +273,8 @@ class OptimizationResult(ObjectJSONEncoder):
         self._save_fig(fig, path=path, show_plots=show_plots)
 
     @timeit
-    def plot_traces(self, path: Path = None, show_plots: bool = True):
-        """Plots optimization traces.
+    def plot_traces(self, path: Path = None, show_plots: bool = True) -> None:
+        """Plot optimization traces.
 
         Optimization time course of costs.
         """
@@ -430,8 +437,8 @@ class OptimizationResult(ObjectJSONEncoder):
                     ax.set_yscale("log")
 
         # correct scatter limits
-        for kx, pidx in enumerate(pids):
-            for ky, pidy in enumerate(pids):
+        for kx, _pidx in enumerate(pids):
+            for ky, _pidy in enumerate(pids):
                 if kx < ky:
                     axes[ky][kx].set_xlim(axes[kx][ky].get_xlim())
                     axes[ky][kx].set_ylim(axes[kx][ky].get_ylim())
