@@ -1,3 +1,5 @@
+"""Optimization of parameter fitting problem."""
+
 import logging
 import time
 from collections import defaultdict
@@ -5,7 +7,7 @@ from copy import deepcopy
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
-from typing import Dict, Iterable, List, Set, Tuple
+from typing import Collection, Dict, Iterable, List, Set, Sized, Tuple
 
 import numpy as np
 import pandas as pd
@@ -16,7 +18,7 @@ from scipy import interpolate, optimize
 
 from sbmlsim.data import Data
 from sbmlsim.experiment import ExperimentRunner
-from sbmlsim.fit.objects import FitExperiment, FitParameter
+from sbmlsim.fit.objects import FitExperiment, FitMapping, FitParameter
 from sbmlsim.fit.sampling import SamplingType, create_samples
 from sbmlsim.model import RoadrunnerSBMLModel
 from sbmlsim.simulation import TimecourseSim
@@ -30,7 +32,9 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class RuntimeErrorOptimizeResult:
-    status: str = -1
+    """Error in opimization."""
+
+    status: str = "-1"
     success: bool = False
     duration: float = -1.0
     cost: float = np.Inf
@@ -38,10 +42,28 @@ class RuntimeErrorOptimizeResult:
 
 
 class OptimizerType(Enum):
+    """Type of optimization.
+
+    Least square
+    """
+
     LEAST_SQUARE = 1
     DIFFERENTIAL_EVOLUTION = 2
 
 
+class FittingType(Enum):
+    """Type of fitting (absolute changes or relative changes to baseline).
+
+    Decides how to fit the data. If the various datasets have large offsets
+    a fitting of the relative changes to baseline can work.
+    As baseline the simulations should contain a pre-simulation.
+    """
+
+    ABSOLUTE_VALUES = 1  # absolute values are use
+    ABSOLUTE_CHANGES_BASELINE = 2  # relative values in respect to baseline are used
+    RELATIVE_CHANGES_BASELINE = 3  # relative values in respect to baseline are used
+
+
 class WeightingLocalType(Enum):
     """Weighting of the data points within a single fit mapping.
 
@@ -51,11 +73,13 @@ class WeightingLocalType(Enum):
 
     NO_WEIGHTING = 1  # data points are weighted equally
     ABSOLUTE_ONE_OVER_WEIGHTING = 2  # data points are weighted as 1/(error-min(error))
-    RELATIVE_ONE_OVER_WEIGHTING = 2  # data points are weighted as 1/(error-min(error))
+    RELATIVE_ONE_OVER_WEIGHTING = 3  # FIXME: check that this is working and documented
 
 
 class ResidualType(Enum):
-    """How are the residuals calculated? Are the absolute residuals used,
+    """Handling of the residuals.
+
+    How are the residuals calculated? Are the absolute residuals used,
     or are the residuals normalized based on the data points, i.e., relative
     residuals.
 
@@ -75,8 +99,8 @@ class OptimizationProblem(object):
     def __init__(
         self,
         opid,
-        fit_experiments: Iterable[FitExperiment],
-        fit_parameters: Iterable[FitParameter],
+        fit_experiments: Collection[FitExperiment],
+        fit_parameters: Collection[FitParameter],
         base_path=None,
         data_path=None,
     ):
@@ -89,8 +113,9 @@ class OptimizationProblem(object):
         :param fit_experiments:
         :param fit_parameters:
         """
-        self.opid = opid
-        self.fit_experiments = FitExperiment.reduce(fit_experiments)
+        self.opid: str = opid
+        # self.fit_experiments = FitExperiment.reduce(fit_experiments)
+        self.fit_experiments = fit_experiments
         self.parameters = fit_parameters
         if self.parameters is None or len(self.parameters) == 0:
             logger.error(
@@ -110,11 +135,15 @@ class OptimizationProblem(object):
         self.base_path = base_path
         self.data_path = data_path
 
-    def __repr__(self):
+    def __repr__(self) -> str:
+        """Get representation."""
         return f"<OptimizationProblem: {self.opid}>"
 
-    def __str__(self):
-        """String representation."""
+    def __str__(self) -> str:
+        """Get string representation.
+
+        This can be run before initialization.
+        """
         info = []
         info.append("-" * 80)
         info.append(f"{self.__class__.__name__}: {self.opid}")
@@ -123,12 +152,39 @@ class OptimizationProblem(object):
         info.extend([f"\t{e}" for e in self.fit_experiments])
         info.append("Parameters")
         info.extend([f"\t{p}" for p in self.parameters])
-        info.append("-" * 80)
-        return "\n".join(info) + "\n"
+        return "\n".join(info)
 
     def report(self, path: Path = None, print_output: bool = True) -> str:
-        """Print and write report."""
-        info = self.__str__()
+        """Print and write report.
+
+        Can only be called after initialization.
+        """
+        core_info = self.__str__()
+        all_info = [
+            core_info,
+            "Settings",
+            f"\tfitting_type: {self.fitting_type}",
+            f"\tresidual_type: {self.residual_type}",
+            f"\tweighting local: {self.weighting_local}",
+            "Data",
+        ]
+        for key in [
+            "experiment_keys",
+            "mapping_keys",
+            "xid_observable",
+            "yid_observable",
+            "x_references",
+            "y_references",
+            "y_errors",
+            "y_errors_type",
+            "weights",
+            "weights_points",
+            "weights_curve",
+        ]:
+            all_info.append(f"\t{key}: {str(getattr(self, key))}")
+
+        info = "\n".join(all_info)
+
         if print_output:
             print(info)
         if path:
@@ -137,16 +193,28 @@ class OptimizationProblem(object):
         return info
 
     def initialize(
-        self, weighting_local: WeightingLocalType, residual_type: ResidualType
+        self,
+        fitting_type: FittingType,
+        weighting_local: WeightingLocalType,
+        residual_type: ResidualType,
     ):
+        """Initialize Optimization problem."""
         # weighting in fitting and handling of residuals
         if weighting_local is None:
             raise ValueError("'weighting_local' is required.")
         if residual_type is None:
             raise ValueError("'residual_type' is required.")
+        if fitting_type is None:
+            logger.warning(
+                "No FittingType provided for fitting, defaulting to absolute"
+            )
+            fitting_type = FittingType.ABSOLUTE_VALUES
+
+        logger.debug(f"fitting_type: {fitting_type}")
         logger.debug(f"weighting_local: {weighting_local}")
         logger.debug(f"residual_type: {residual_type}")
 
+        self.fitting_type = fitting_type
         self.weighting_local = weighting_local
         self.residual_type = residual_type
 
@@ -168,8 +236,9 @@ class OptimizationProblem(object):
         self.y_references = []
         self.y_errors = []
         self.y_errors_type = []
-        self.weights_local = []  # weights for data points
-        self.weights_global_user = []  # user defined weights per mapping
+        self.weights = []  # total weights for points (errors + user)
+        self.weights_points = []  # weights for data points based on errors
+        self.weights_curve = []  # user defined weights per mapping/curve
 
         self.models = []
         self.xmodel = np.empty(shape=(len(self.pids)))
@@ -177,18 +246,18 @@ class OptimizationProblem(object):
         self.selections = []
 
         # Collect information for simulations
-        for fit_experiment in self.fit_experiments:
+        for fit_experiment in self.fit_experiments:  # type: FitExperiment
 
             # get simulation experiment
             sid = fit_experiment.experiment_class.__name__
             sim_experiment = self.runner.experiments[sid]
 
             # FIXME: selections should be based on fit mappings
-            selections = set()
+            selections_set: Set[str] = set()
             for d in sim_experiment._data.values():  # type: Data
                 if d.is_task():
-                    selections.add(d.index)
-            selections = list(selections)
+                    selections_set.add(d.index)
+            selections: List[str] = list(selections_set)
 
             # use all fit_mappings if None are provided
             if fit_experiment.mappings is None:
@@ -197,17 +266,16 @@ class OptimizationProblem(object):
 
             # collect information for single mapping
             for k, mapping_id in enumerate(fit_experiment.mappings):
-                # user defined weight of fit mapping
-                weight_global_user = fit_experiment.weights[k]
-
                 # sanity checks
                 if mapping_id not in sim_experiment._fit_mappings:
                     raise ValueError(
                         f"Mapping key '{mapping_id}' not defined in "
-                        f"SimulationExperiment '{sim_experiment}'."
+                        f"SimulationExperiment\n"
+                        f"{sim_experiment}\n"
+                        f"{fit_experiment}"
                     )
 
-                mapping = sim_experiment._fit_mappings[mapping_id]  # type: FitMapping
+                mapping: FitMapping = sim_experiment._fit_mappings[mapping_id]
 
                 if mapping.observable.task_id is None:
                     raise ValueError(
@@ -220,6 +288,26 @@ class OptimizationProblem(object):
                         f"'{mapping.reference}'"
                     )
 
+                # get weight for curve
+                if fit_experiment.use_mapping_weights:
+                    # use provided mapping weights
+                    weight_curve = mapping.weight
+                    fit_experiment.weights[k] = weight_curve
+                    if weight_curve is None:
+                        raise ValueError(
+                            f"If `use_mapping_weights` is set on a FitExperiment "
+                            f"then all mappings must have a weight. But "
+                            f"weight '{weight_curve}' in {mapping}."
+                        )
+                else:
+                    weight_curve = fit_experiment.weights[k]
+
+                if weight_curve < 0:
+                    raise ValueError(
+                        f"Mapping weights must be positive but "
+                        f"weight '{mapping.weight}' in {mapping}"
+                    )
+
                 task_id = mapping.observable.task_id
                 task = sim_experiment._tasks[task_id]
                 model = sim_experiment._models[
@@ -229,7 +317,7 @@ class OptimizationProblem(object):
 
                 if not isinstance(simulation, TimecourseSim):
                     raise ValueError(
-                        f"Only TimecourseSims supported in fitting: " f"'{simulation}"
+                        f"Only TimecourseSims supported in fitting: '{simulation}"
                     )
 
                 # observable units
@@ -259,6 +347,12 @@ class OptimizationProblem(object):
                 x_ref = data_ref.x.magnitude
                 y_ref = data_ref.y.magnitude
 
+                if self.fitting_type == FittingType.ABSOLUTE_CHANGES_BASELINE:
+                    # Use absolute changes to baseline, which is the first point
+                    y_ref = (
+                        y_ref - y_ref[0]
+                    )  # FIXME: will break if first data point is bad;
+
                 # Use errors for weighting (tries SD and falls back on SE)
                 # FIXME: SE & SD not handled uniquely, i.e., slightly different weighting
                 y_ref_err = None
@@ -283,6 +377,7 @@ class OptimizationProblem(object):
                         y_ref_err[np.isnan(y_ref_err)] = np.nanmax(y_ref_err)
 
                 # remove zero values for relative errors (no inf residuals)
+                # FIXME: check how this works with the relative changes (due to 0.0 at first data point)
                 if self.residual_type == ResidualType.RELATIVE_RESIDUALS:
                     nonzero_mask = y_ref != 0.0
                     if not np.all(nonzero_mask):
@@ -321,28 +416,31 @@ class OptimizationProblem(object):
                             f"{fit_experiment}.{mapping_id}: NaN or INF in y_ref_err: {y_ref_err}"
                         )
 
-                # calculate local weights based on errors
-                weights = np.ones_like(y_ref)  # local weights are by default 1.0
+                # FIXME: relative_changes to baseline errors;
 
-                if self.weighting_local != WeightingLocalType.NO_WEIGHTING:
+                # --- WEIGHTS ---
+                # calculate local weights based on errors (i.e. weights for data
+                # points
+                # FIXME: the weights must be normalized for a mapping based on data points;
+                # FIXME: also make sure that data points with error contribute
+                # correctly compared to no errors
+                weight_points = np.ones_like(y_ref)  # local weights are by default 1.0
+                if self.weighting_local == WeightingLocalType.NO_WEIGHTING:
+                    pass
+                else:
                     if y_ref_err is not None:
                         if (
                             self.weighting_local
                             == WeightingLocalType.ABSOLUTE_ONE_OVER_WEIGHTING
                         ):
-                            weights = (
-                                1.0 / y_ref_err
-                            )  # the larger the error, the smaller the weight
-                            weights = weights / np.max(
-                                weights
-                            )  # normalize maximal weight to 1.0, weights in (0, 1]
+                            # the larger the error, the smaller the weight
+                            weight_points = 1.0 / y_ref_err
                         elif (
                             self.weighting_local
                             == WeightingLocalType.RELATIVE_ONE_OVER_WEIGHTING
                         ):
-                            weights = (
-                                y_ref / y_ref_err
-                            )  # the larger the error, the smaller the weight
+                            # the larger the error, the smaller the weight
+                            weight_points = y_ref / y_ref_err
                         else:
                             raise ValueError(
                                 f"Local weighting not supported: {self.weighting_local}"
@@ -353,13 +451,18 @@ class OptimizationProblem(object):
                             f"no errors in reference data."
                         )
 
-                if False:
-                    print("-" * 80)
-                    print(f"{fit_experiment}.{mapping_id}")
-                    print(f"weights: {weights}")
-                    print(f"y_ref: {y_ref}")
-                    print(f"y_ref_err: {y_ref_err}")
+                # normalize weights to mean=1.0 for given curve
+                # this makes the weights comparable
+                weight_points = weight_points / np.mean(weight_points)
 
+                # apply local weighting & user defined weighting
+                # (in the cost function the weighted residuals are squared)
+                # sum(w_i * r_i^2) = sum((w_i^0.5*r_i)^2)
+                weight = np.sqrt(weight_points * weight_curve)
+                if np.any(weight < 0):
+                    raise ValueError("Negative weights encountered.")
+
+                # --- STORE INITIAL PARAMETERS ---
                 # store initial model parameters
                 for k, pid in enumerate(self.pids):
                     pid_value = model.r[pid]
@@ -385,11 +488,25 @@ class OptimizationProblem(object):
                 self.y_references.append(y_ref)
                 self.y_errors.append(y_ref_err)
                 self.y_errors_type.append(y_ref_err_type)
-                self.weights_local.append(weights)
-                self.weights_global_user.append(weight_global_user)
+                self.weights_points.append(weight_points)
+                self.weights_curve.append(weight_curve)
+                self.weights.append(weight)
+
+                # debug info
+                if False:
+                    print("-" * 80)
+                    print(f"{fit_experiment}.{mapping_id}")
+                    print(f"weight: {weight}")
+                    print(f"weight_curve: {weight_curve}")
+                    print(f"weight_points: {weight_points}")
+                    print(f"y_ref: {y_ref}")
+                    print(f"y_ref_err: {y_ref_err}")
+
+            # Print mappings with calculated weights
+            print(fit_experiment)
 
     def set_simulator(self, simulator):
-        """Sets the simulator on the runner and the experiments.
+        """Set the simulator on the runner and the experiments.
 
         :param simulator:
         :return:
@@ -403,6 +520,7 @@ class OptimizationProblem(object):
         verbose=False,
         optimizer: OptimizerType = OptimizerType.LEAST_SQUARE,
         sampling: SamplingType = SamplingType.UNIFORM,
+        fitting_type: FittingType = FittingType.ABSOLUTE_VALUES,
         weighting_local: WeightingLocalType = WeightingLocalType.NO_WEIGHTING,
         residual_type: ResidualType = ResidualType.ABSOLUTE_RESIDUALS,
         variable_step_size=True,
@@ -410,8 +528,10 @@ class OptimizationProblem(object):
         absolute_tolerance=1e-8,
         **kwargs,
     ) -> Tuple[List[scipy.optimize.OptimizeResult], List]:
-        """Run parameter optimization"""
+        """Run parameter optimization."""
+
         # additional settings for optimization
+        self.fitting_type = fitting_type
         self.weighting_local = weighting_local
         self.residual_type = residual_type
         self.variable_step_size = variable_step_size
@@ -451,9 +571,9 @@ class OptimizationProblem(object):
 
     @timeit
     def _optimize_single(
-        self, x0=None, optimizer=OptimizerType.LEAST_SQUARE, **kwargs
+        self, x0: np.ndarray = None, optimizer=OptimizerType.LEAST_SQUARE, **kwargs
     ) -> Tuple[scipy.optimize.OptimizeResult, List]:
-        """Runs single optimization with x0 start values.
+        """Run single optimization with x0 start values.
 
         :param x0: parameter start vector (important for deterministic optimizers)
         :param optimizer: optimization algorithm and method
@@ -522,12 +642,13 @@ class OptimizationProblem(object):
         else:
             raise ValueError(f"optimizer is not supported: {optimizer}")
 
-    def cost_least_square(self, xlog):
+    def cost_least_square(self, xlog: np.ndarray) -> float:
+        """Get least square costs for parameters."""
         res_weighted = self.residuals(xlog)
         return 0.5 * np.sum(np.power(res_weighted, 2))
 
-    def residuals(self, xlog, complete_data=False):
-        """Calculates residuals for given parameter vector.
+    def residuals(self, xlog: np.ndarray, complete_data=False):
+        """Calculate residuals for given parameter vector.
 
         :param x: logarithmic parameter vector
         :param complete_data: boolean flag to return additional information
@@ -538,6 +659,8 @@ class OptimizationProblem(object):
         # see https://github.com/scipy/scipy/issues/7632
         # print(f"\t{xlog}")
         x = np.power(10, xlog)
+
+        # FIXME: handle parts better
         parts = []
         if complete_data:
             residual_data = defaultdict(list)
@@ -546,10 +669,10 @@ class OptimizationProblem(object):
         simulator = self.runner.simulator  # type: SimulatorSerial
         Q_ = self.runner.Q_
 
-        for k, mapping_id in enumerate(self.mapping_keys):
-            # Overwrite initial changes in the simulation
+        for k, _ in enumerate(self.mapping_keys):
+            # update initial changes
             changes = {
-                self.pids[i]: Q_(value, self.punits[i]) for i, value in enumerate(x)
+                self.pids[ix]: Q_(value, self.punits[ix]) for ix, value in enumerate(x)
             }
             self.simulations[k].timecourses[0].changes.update(changes)
 
@@ -564,9 +687,8 @@ class OptimizationProblem(object):
             simulation.normalize(udict=simulator.udict, ureg=simulator.ureg)
 
             # run simulation
-            # logger.warning(f"Running simulation: {k} - {self.experiment_keys[k]} - {mapping_id}")
-
             try:
+                # FIXME: just simulate at the requested timepoints with step
                 df = simulator._timecourses([simulation])[0]
 
                 # interpolation of simulation results
@@ -577,6 +699,8 @@ class OptimizationProblem(object):
                     assume_sorted=True,
                 )
                 y_obsip = f(self.x_references[k])
+                if self.fitting_type == FittingType.ABSOLUTE_CHANGES_BASELINE:
+                    y_obsip = y_obsip - y_obsip[0]
 
                 # calculate absolute & relative residuals
                 res_abs = y_obsip - self.y_references[k]
@@ -605,22 +729,11 @@ class OptimizationProblem(object):
             elif self.residual_type == ResidualType.RELATIVE_RESIDUALS:
                 res = res_rel
 
-            # apply local weighting & user defined weighting (in the cost function the weighted residuals are squared)
-            # sum(w_i * r_i^2) = sum((w_i^0.5*r_i)^2)
-            resw = (
-                res
-                * np.sqrt(self.weights_local[k])
-                * np.sqrt(self.weights_global_user[k])
-            )  # FIXME: handle square of global weights correctly
+            resw = res * self.weights[k]
             parts.append(resw)
 
-            # if False:
-            #    print("residuals:", self.experiment_keys[k], 'mapping_id:', mapping_id, res)
-            #    print("residuals weighted:", self.experiment_keys[k], 'mapping_id:', mapping_id, resw)
-
             # for post_processing
             if complete_data:
-
                 residual_data["x_obs"].append(df[self.xid_observable[k]])
                 residual_data["y_obs"].append(df[self.yid_observable[k]])
                 residual_data["y_obsip"].append(y_obsip)
@@ -637,6 +750,7 @@ class OptimizationProblem(object):
         else:
             res_all = np.concatenate(parts)
             # store the local step
+            # FIXME
             self._trajectory.append((deepcopy(x), 0.5 * np.sum(np.power(res_all, 2))))
             return res_all
 
@@ -653,6 +767,7 @@ class OptimizationAnalysis:
             plt.show()
         if path is not None:
             fig.savefig(path, bbox_inches="tight")
+        plt.close(fig)
 
     @timeit
     def plot_fits(self, x, path: Path = None, show_plots: bool = True):
@@ -740,7 +855,7 @@ class OptimizationAnalysis:
             # global reference data
             sid = self.op.experiment_keys[k]
             mapping_id = self.op.mapping_keys[k]
-            weights = self.op.weights_local[k]
+            # weights = self.op.weights_points[k]
             x_ref = self.op.x_references[k]
             y_ref = self.op.y_references[k]
             y_ref_err = self.op.y_errors[k]
@@ -760,7 +875,7 @@ class OptimizationAnalysis:
                 res = res_data["residuals"][k]
                 res_weighted = res_data["residuals_weighted"][k]
                 res_abs = res_data["res_abs"][k]
-                res_rel = res_data["res_rel"][k]
+                # res_rel = res_data["res_rel"][k]
 
                 cost = res_data["cost"][k]
 
@@ -863,8 +978,11 @@ class OptimizationAnalysis:
                 )
 
     @timeit
-    def plot_costs(self, x, path: Path = None, show_plots: bool = True):
-        """Plot cost function comparison"""
+    def plot_costs(self, x, path: Path = None, show_plots: bool = True) -> pd.DataFrame:
+        """Plot cost function comparison.
+
+        # FIXME: separate calculation of cost DataFrame
+        """
         xparameters = {
             # model parameters
             "model": self.op.xmodel,
@@ -878,7 +996,7 @@ class OptimizationAnalysis:
         for key, xpar in xparameters.items():
             res_data = self.op.residuals(xlog=np.log10(xpar), complete_data=True)
             costs[key] = res_data["cost"]
-            for k, mapping_key in enumerate(self.op.mapping_keys):
+            for k, _ in enumerate(self.op.mapping_keys):
                 data.append(
                     {
                         "id": f"{self.op.experiment_keys[k]}_{self.op.mapping_keys[k]}",
@@ -928,9 +1046,9 @@ class OptimizationAnalysis:
             alpha=0.8,
         )
 
-        for k, mapping_key in enumerate(self.op.mapping_keys):
+        for k, exp_key in enumerate(self.op.experiment_keys):
             ax.annotate(
-                f"{self.op.experiment_keys[k]}",
+                exp_key,
                 xy=(
                     costs["model"][k],
                     costs["fit"][k],
