@@ -1,9 +1,7 @@
-"""
-Sampling of parameter values
-"""
+"""Sampling of parameter values."""
 import logging
 from enum import Enum
-from typing import List
+from typing import Dict, Iterable, List
 
 import numpy as np
 import pandas as pd
@@ -18,6 +16,11 @@ logger = logging.getLogger(__name__)
 
 
 class SamplingType(Enum):
+    """Type of sampling used.
+
+    The LHS options are latin hypercube sampling types.
+    """
+
     LOGUNIFORM = 1
     UNIFORM = 2
     LOGUNIFORM_LHS = 3
@@ -25,7 +28,7 @@ class SamplingType(Enum):
 
 
 def create_samples(
-    parameters: List[FitParameter],
+    parameters: Iterable[FitParameter],
     size,
     sampling=SamplingType.LOGUNIFORM,
     seed=None,
@@ -94,7 +97,7 @@ def create_samples(
 
 
 def plot_samples(samples):
-    """Plot samples"""
+    """Plot samples."""
     df = list(samples.values())[0]
     pids = df.columns
 
@@ -126,13 +129,17 @@ def plot_samples(samples):
     plt.show()
 
 
-def example1():
-    parameters = [
+def example_sampling():
+    """Examples howing the use of the sampling.
+
+    :return:
+    """
+    parameters: List[FitParameter] = [
         FitParameter(pid="p1", lower_bound=10, upper_bound=1e4),
         FitParameter(pid="p2", lower_bound=1, upper_bound=1e3),
         FitParameter(pid="p3", lower_bound=1, upper_bound=1e3),
     ]
-    samples = {}
+    samples: Dict[str, pd.DataFrame] = {}
     for sampling in [
         SamplingType.UNIFORM,
         SamplingType.UNIFORM_LHS,
@@ -150,4 +157,4 @@ def example1():
 
 
 if __name__ == "__main__":
-    example1()
+    example_sampling()
