@@ -1,11 +1,9 @@
-"""
-Definition of timecourse simulations and timecourse definitions.
-"""
+"""Definition of timecourses and timecourse simulations."""
 import json
 import logging
 from copy import deepcopy
 from pathlib import Path
-from typing import Dict, List, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
 
@@ -41,6 +39,9 @@ class Timecourse(ObjectJSONEncoder):
     ):
         """Create a time course definition for simulation.
 
+        Discarded simulations do not add time shifts, i.e. a pre-simulation
+        does not increase the time of simulation.
+
         :param start: start time
         :param end: end time
         :param steps: simulation steps
@@ -65,40 +66,47 @@ class Timecourse(ObjectJSONEncoder):
         self.model_manipulations = deepcopy(model_manipulations)
         self.discard = discard
 
-    def __repr__(self):
+    def __repr__(self) -> str:
+        """Get representation."""
         return f"Timecourse([{self.start}:{self.end}])"
 
     def to_dict(self):
-        """ Convert to dictionary. """
+        """Convert to dictionary."""
         d = dict()
         for key in self.__dict__:
             d[key] = self.__dict__[key]
         return d
 
-    def add_change(self, sid: str, value: float):
+    def add_change(self, sid: str, value: float) -> None:
+        """Add change."""
         self.changes[sid] = value
 
-    def remove_change(self, sid: str):
+    def remove_change(self, sid: str) -> None:
+        """Remove change for given id."""
         del self.changes[sid]
 
-    def add_model_change(self, sid: str, change):
+    def add_model_change(self, sid: str, change) -> None:
+        """Add model change."""
         self.model_changes[sid] = change
 
-    def add_model_changes(self, model_changes):
+    def add_model_changes(self, model_changes: Dict[str, str]) -> None:
+        """Add model changes."""
         self.model_changes.update(model_changes)
 
-    def remove_model_change(self, sid: str):
+    def remove_model_change(self, sid: str) -> None:
+        """Remove model change for id."""
         del self.model_changes[sid]
 
     def normalize(self, udict, ureg):
-        """ Normalize values to model units for all changes."""
+        """Normalize values to model units for all changes."""
         self.model_changes = Units.normalize_changes(
             self.model_changes, udict=udict, ureg=ureg
         )
         self.changes = Units.normalize_changes(self.changes, udict=udict, ureg=ureg)
 
     def strip_units(self):
-        """Stripping units for parallel simulation.
+        """Strip units for parallel simulation.
+
         All changes must be normalized before stripping !.
         """
         self.changes = {k: v.magnitude for k, v in self.changes.items()}
@@ -113,11 +121,12 @@ class TimecourseSim(AbstractSim):
     def __init__(
         self,
         timecourses: List[Timecourse],
-        selections: list = None,
+        selections: Optional[List[str]] = None,
         reset: bool = True,
         time_offset: float = 0.0,
     ):
-        """
+        """Initialize timecourse sim.
+
         :param timecourses:
         :param selections:
         :param reset: complete reset of model
@@ -128,11 +137,16 @@ class TimecourseSim(AbstractSim):
 
         self.timecourses = []
         for tc in timecourses:
+            if not tc:
+                # remove empty elements (allows for cleaner syntax)
+                continue
+
             if isinstance(tc, dict):
                 # construct from dict
-                self.timecourses.append(Timecourse(**tc))
-            else:
-                self.timecourses.append(tc)
+                tc = Timecourse(**tc)
+
+            # make a copy to ensure independence of instances
+            self.timecourses.append(deepcopy(tc))
 
         if len(self.timecourses) == 0:
             logger.error("No timecourses in simulation")
@@ -149,23 +163,26 @@ class TimecourseSim(AbstractSim):
 
         self.time = self._time()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
+        """Get representation."""
         return f"TimecourseSim({[tc for tc in self.timecourses]})"
 
-    def _time(self):
-        """Calculates the time vector complete simulation."""
+    def _time(self) -> np.ndarray:
+        """Calculate the time vector complete simulation."""
         t_offset = self.time_offset
         time_vecs = []
         for tc in self.timecourses:
             time_vecs.append(np.linspace(tc.start, tc.end, num=tc.steps + 1) + t_offset)
             t_offset += tc.end
-        return np.concatenate(time_vecs)
+        res: np.ndarray = np.concatenate(time_vecs)
+        return res
 
     def dimensions(self) -> List[Dimension]:
+        """Get dimensions."""
         return [Dimension(dimension="time", index=self.time)]
 
     def add_model_changes(self, model_changes: Dict) -> None:
-        """Adds model changes to given simulation. """
+        """Add model changes to given simulation."""
         if self.timecourses:
             tc = self.timecourses[0]  # type: Timecourse
             tc.add_model_changes(model_changes)
@@ -181,7 +198,7 @@ class TimecourseSim(AbstractSim):
             tc.strip_units()
 
     def to_dict(self):
-        """ Convert to dictionary. """
+        """Convert to dictionary."""
         d = {
             "type": self.__class__.__name__,
             "selections": self.selections,
@@ -211,6 +228,7 @@ class TimecourseSim(AbstractSim):
             d.pop("type")  # serialized property
         return TimecourseSim(**d)
 
-    def __str__(self):
+    def __str__(self) -> str:
+        """Get string representation."""
         lines = ["-" * 40, f"{self.__class__.__name__}", "-" * 40, self.to_json()]
         return "\n".join(lines)
