@@ -1,6 +1,4 @@
-"""
-Testing DataSet and Data functionality.
-"""
+"""Testing DataSet and Data functionality."""
 import pandas as pd
 
 from sbmlsim.data import DataSet, load_pkdb_dataframe
