@@ -68,6 +68,37 @@ def test_timecourse_combined():
     assert s.time.values[-1] == 250.0
 
 
+def test_timecourse_concat():
+    """Reuse of timecourses."""
+    simulator = Simulator(MODEL_REPRESSILATOR)
+    tc = Timecourse(start=0, end=50, steps=100, changes={"X": 10})
+
+    s = simulator._timecourse(simulation=TimecourseSim([tc] * 3))
+    assert isinstance(s, pd.DataFrame)
+    assert "time" in s
+    assert s.time.values[-1] == 150.0
+    assert len(s) == 3 * 101
+    assert s.X.values[0] == 10.0
+    assert s.X.values[101] == 10.0
+    assert s.X.values[202] == 10.0
+
+
+def test_timecourse_empty():
+    """Reuse of timecourses."""
+    simulator = Simulator(MODEL_REPRESSILATOR)
+    tc = Timecourse(start=0, end=50, steps=100, changes={"X": 10})
+
+    tcsim = TimecourseSim([None, tc, None])
+    s = simulator._timecourse(
+        simulation=tcsim,
+    )
+    assert len(tcsim.timecourses) == 1
+    assert isinstance(s, pd.DataFrame)
+    assert "time" in s
+    assert s.time.values[-1] == 50.0
+    assert len(s) == 101
+
+
 def test_timecourse_discard():
     """Test discarding pre-simulation."""
     simulator = Simulator(MODEL_REPRESSILATOR)
@@ -93,5 +124,5 @@ def test_timecourse_discard():
     assert isinstance(s, pd.DataFrame)
     assert "time" in s
     assert len(s.time) == 101
-    assert s.time.values[0] == 100.0
-    assert s.time.values[-1] == 200.0
+    assert s.time.values[0] == 0.0
+    assert s.time.values[-1] == 100.0
